package de.fraunhofer.iem.dataprovider.taskManager.tasks

import de.fraunhofer.iem.dataprovider.gitlab.GitConfiguration
import de.fraunhofer.iem.dataprovider.gitlab.Platform
import de.fraunhofer.iem.dataprovider.gitlab.RepositoryRepository
import de.fraunhofer.iem.dataprovider.taskManager.Event
import de.fraunhofer.iem.dataprovider.taskManager.GetGitlabProjectDone
import org.gitlab4j.api.GitLabApi


class GetGitlabProjectTask(
    private val repoId: Long, private val gitlabConfiguration: GitConfiguration,
    override val responseChannel: suspend (task: Event) -> Unit,
    private val repository: RepositoryRepository
) : Task() {


    private val gitlabApi: GitLabApi = GitLabApi(gitlabConfiguration.host, gitlabConfiguration.accessToken)

    override suspend fun execute() {
        logger.info("Gitlab config ${gitlabConfiguration.host} ${gitlabConfiguration.accessToken}")

        val project = gitlabApi.projectApi.getProject(repoId)
        logger.info(project.toString())
        val projectUri = project.httpUrlToRepo

        val gitRepository = GitRepository(project.path, projectUri, repoId, Platform.OPEN_CODE)

        var repo = repository.findByRepoIdAndPlatform(gitRepository.id, gitRepository.platform)
        if (repo == null) {
            logger.info("Repository ${gitRepository.id} ${gitRepository.platform} doesn't exist in database.")
            repo = repository.save(gitRepository.toDbObject())
        }

        logger.info("Retrieved project ${project.path} and url $projectUri")

        responseChannel(GetGitlabProjectDone(taskID, repo.id!!, gitRepository))
    }
}
